/**
 *  Plugin for grouped column headers
 *  @class Ext.ux.plugins.GroupHeaderGrid
 * 	@extends Ext.util.Observable
 *  @author yvind Neshaug
 *  
 *  Based on the code in this thread: http://extjs.com/forum/showthread.php?t=12677
 *  Thanks to JEBriggs for starting up the code and resizing code from Clayton
 *  Thanks to JorisA for the idea of making this a plugin instead of an extension.
 *  
 *  Supports infinte number of grouped header rows
 *  Supports header group spanning of rows
 *  
 */
Ext.namespace("Ext.ux.plugins");

Ext.ux.plugins.GroupHeaderGrid = function(config) {
	Ext.apply(this, config);
};
Ext.extend(Ext.ux.plugins.GroupHeaderGrid, Ext.util.Observable, {
	// PLUGIN INIT
	init: function(grid){
		this.grid = grid;
		this.view = this.grid.getView();
		this.cm = this.grid.getColumnModel();
		this.initTemplates();
		this.view.renderHeaders.createInterceptor(this.renderHeaders(),this);
	},
	initTemplates: function(){
		this.view.initTemplates.call(this);
		if(!this.templates.groupedHeader){
		    this.templates.groupedHeader = new Ext.Template(
	             '<table border="0" cellspacing="0" cellpadding="0" style="{tstyle}">',
	             '<thead>{groupRows}',
	             '<tr class="x-grid3-hd-row">{cells}</tr></thead>',
	             '</table>'
        	);
    	}
		if (!this.templates.groupRow){
			this.templates.groupRow = new Ext.Template(
				'<tr class="x-grid3-hd-row">{groupCells}</tr>'
			);
		}     
	    if(!this.templates.groupCell){
		    this.templates.groupCell = new Ext.Template(
				 '<td id="{grId}" class="{cellClass}" rowspan="{rowspan}" colspan="{colspan}"><div class="x-grid3-hd-inner" unselectable="on" style="text-align:{align}">{header}</div></td>'
	        );
	    }
	},
	renderHeaders : function(cm){
		if (cm){
			this.cm = cm;
		}
		if(!this.cm.rows) {
			return true;
		}
		else {
			this.view.renderHeaders = this.renderGroupedHeaders.createDelegate(this);
			
			this.view.getHeaderCell = this.getHeaderCell.createDelegate(this);
			this.view.updateSortIcon = this.updateSortIcon.createDelegate(this);
			this.view.getColumnWidth = this.getColumnWidth.createDelegate(this);
			this.view.getColumnStyle = this.getColumnStyle.createDelegate(this);
			this.view.getTotalWidth = this.getTotalWidth.createDelegate(this);
			this.view.updateColumnHidden=this.updateColumnHidden.createDelegate(this);
			
			this.view.renderHeaders();
			this.colModel = this.cm;
			this.headerBandLinks = this.linkBandsAndHeaders();
			
			return false;
		}
	},
	renderGroupedHeaders : function() {
		var cm = this.cm;
		var rows = this.cm.rows;
	    var groups;
		
        var cellTemplate = this.templates.hcell;

        var cellMarkup = [], sb = [], cellParams = {};
        var groupCellMarkup = [];
		var rowsMarkup = [];
		
		for (var i = 0; i < rows.length; i++) {//create markup for rows
			groups = rows[i];
			for (var j = 0; j < groups.length; j++) {//create markup for group cells
				groups[j].id="ext-ux-grhd-gen-"+i+"."+j;
				groupCellMarkup[groupCellMarkup.length] = this.renderGroupCell(groups[j]);
			}
			rowsMarkup[rowsMarkup.length] = this.renderGroupRows(groupCellMarkup.join(""));
			var groupCellMarkup = [];
		}
        for(var i = 0, len = cm.getColumnCount(); i < len; i++){ // create markup for leaf cells
        	cellMarkup[cellMarkup.length] = this.renderHeaderCell(cm, i);
        }
        // use a different template
        return this.templates.groupedHeader.apply({groupRows: rowsMarkup.join(""), cells: cellMarkup.join(""), tstyle:'width:'+this.view.getTotalWidth()+';'});
	},
	renderGroupCell : function(group,groupId) {
		var template = this.templates.groupCell;
		var cellClass;
		var rowspan;
		if (group.colspan < 2 || group.colspan === undefined){
			cellClass = "ux-grid-hd-group-nocolspan-cell";
		}
		else {
			cellClass = "ux-grid-hd-group-cell";
		}
    	return template.apply({
			header: group.header,
			colspan: group.colspan,
			align: group.align,
			rowspan: group.rowspan,
			cellClass: cellClass,
			grId: group.id
		});
	},
	renderGroupRows : function(groupCellMarkup) {
		var template = this.templates.groupRow;
		return template.apply({groupCells: groupCellMarkup});
	},
	renderHeaderCell : function(cm,index){
		var template = this.templates.hcell;
		var params = {};
		params.id = cm.getColumnId(index);
	    params.value = cm.getColumnHeader(index) || "";
	    params.style = this.view.getColumnStyle(index, true);
	    if(cm.config[index].align == 'right'){
	    	params.istyle = 'padding-right:16px';
	    }
	    return template.apply(params);
	},
	// from gridview, with minor fixes
	getHeaderCell : function(index){
	    var hds = this.view.mainHd.select('.x-grid3-cell');
	    return (hds.item(index).dom);
	},
	updateSortIcon : function(col, dir){
	    var sc = this.view.sortClasses;
	    var hds = this.view.mainHd.select('.x-grid3-cell').removeClass(sc);
	    hds.item(col).addClass(sc[dir == "DESC" ? 1 : 0]);
	},
	getColumnWidth : function(col){
        var w = this.cm.getColumnWidth(col);
        if(typeof w == 'number'){
            return (Ext.isBorderBox ? w : (w-this.view.borderWidth > 0 ? w-this.view.borderWidth:0)) + 'px';
        }
        return w;
    },
	getColumnStyle : function(col, isHeader){
        var style = !isHeader ? (this.cm.config[col].css || '') : '';
        style += 'width:'+this.view.getColumnWidth(col)+';';
        if(this.cm.isHidden(col)){
            style += 'display:none;';
        }
        var align = this.cm.config[col].align;
        if(align){
            style += 'text-align:'+align+';';
        }
        return style;
    },
	getTotalWidth : function(){
        return this.cm.getTotalWidth()+'px';
    },
	
	updateColumnHidden : function(col, hidden){
        var tw = this.getTotalWidth();
		var bands = this.headerBandLinks[col]||[];
        this.view.innerHd.firstChild.firstChild.style.width = tw;

        var display = hidden ? 'none' : '';
		
		
        var hd = this.getHeaderCell(col);
        hd.style.display = display;

        var ns = this.view.getRows();
        for(var i = 0, len = ns.length; i < len; i++){
            ns[i].style.width = tw;
            ns[i].firstChild.style.width = tw;
            ns[i].firstChild.rows[0].childNodes[col].style.display = display;
        }

		for(var i = 0, len = bands.length; i < len; i++){
			var bandId = bands[i];
			var band = Ext.getDom(bandId);
			
			if (hidden){
				if (band.colSpan < 2 || band.colSpan === undefined){
					band.style.display = 'none';
				}
				else{
					band.colSpan += -1;
				}
			}
			else{
				if ((band.colSpan < 2 || band.colSpan === undefined)&&band.style.display === 'none'){
					band.style.display = '';
				}
				else{
					band.colSpan += 1;
				}
			}
			
			
		}
		
        this.view.onColumnHiddenUpdated(col, hidden, tw);

        delete this.view.lastViewWidth; // force recalc
        this.view.layout();
    },
	
	linkBandsAndHeaders : function(){
		var cm = this.grid.getColumnModel();
		var columnCount = cm.getColumnCount();
		var headerBands = new Array(columnCount);
		var nrOfRows = cm.rows.length;
		for (var i = 0;i<nrOfRows;i++){
			var headersInRow = cm.rows[i];
			var headersInRowCount = headersInRow.length;
			for (var j = 0;j<headersInRowCount;j++){
				var header = headersInRow[j];
				var headerRowSpan = header.rowspan||1;
				var headerColSpan = header.colspan||1;
				var colend=headerColSpan;
				headersInRow[j].colstart = headersInRow[j].colstart||0;
				headersInRow[j].colend = headersInRow[j].colend||0+headerColSpan;
				for (var m=j+1; m<headersInRowCount;m++){
					headersInRow[m].colstart = headersInRow[m].colstart||0+colend;
					colend += headersInRow[m].colspan||1;
					headersInRow[m].colend = headersInRow[m].colend||0+colend;
				}
				
				for (var k=1; k<headerRowSpan; k++){
					var headersInRow2 = cm.rows[k+i];
					var headersInRowCount2 = headersInRow2.length;
					colend=headerColSpan;
					for (var l = j;l<headersInRowCount2;l++){
						headersInRow2[l].colstart = headersInRow2[l].colstart||0 + colend;
						colend += headersInRow2[l].colspan||1;
						headersInRow2[l].colend = headersInRow2[l].colend||0 + colend;
					}
					
				}
				
			
			}
			
		}
		
		for (var i=0; i<columnCount; i++){
			
			for (var j=0; j<nrOfRows; j++){
				var headersInRow = cm.rows[j];
				var headersInRowCount = headersInRow.length;
				for (var k = 0;k<headersInRowCount;k++){
					if (headersInRow[k].colstart<=i && headersInRow[k].colend>i){
						var bands = headerBands[i]||[];
						bands[bands.length]=headersInRow[k].id;
						headerBands[i]=bands;
					}
				}
			}
			
		}
		
		
		return headerBands;
		
	}
});